@@ -7,7 +7,8 @@ import sys
 from conda_build.conda_interface import ArgumentParser
 
 from conda_build import api
-from conda_build.index import DEFAULT_SUBDIRS, MAX_THREADS_DEFAULT
+from conda_build.index import MAX_THREADS_DEFAULT
+from conda_build.utils import DEFAULT_SUBDIRS
 
 logging.basicConfig(level=logging.INFO)
 
@@ -59,6 +60,19 @@ def parse_args(args):
     p.add_argument(
         "--no-progress", help="Hide progress bars", action="store_false", dest="progress"
     )
+    p.add_argument(
+        "--current-index-versions-file", "-m",
+        help="""
+        YAML file containing name of package as key, and list of versions as values.  The current_index.json
+        will contain the newest from this series of versions.  For example:
+
+        python:
+          - 2.7
+          - 3.6
+
+        will keep python 2.7.X and 3.6.Y in the current_index.json, instead of only the very latest python version.
+        """
+    )
 
     args = p.parse_args(args)
     return p, args
@@ -66,9 +80,11 @@ def parse_args(args):
 
 def execute(args):
     _, args = parse_args(args)
+
     api.update_index(args.dir, check_md5=args.check_md5, channel_name=args.channel_name,
                      threads=args.threads, subdir=args.subdir, patch_generator=args.patch_generator,
-                     verbose=args.verbose, progress=args.progress, hotfix_source_repo=args.hotfix_source_repo)
+                     verbose=args.verbose, progress=args.progress, hotfix_source_repo=args.hotfix_source_repo,
+                     current_index_versions=args.current_index_versions_file)
 
 
 def main():
