@@ -4,9 +4,10 @@ from __future__ import absolute_import, division, print_function, unicode_litera
 from functools import partial
 import os
 from os import lstat
-from pkg_resources import parse_version
 from importlib import import_module
 
+from pkg_resources import parse_version
+
 from conda import __version__ as CONDA_VERSION
 
 
@@ -32,6 +33,7 @@ conda_44 = parse_version(CONDA_VERSION) >= parse_version("4.4.0a0")
 conda_45 = parse_version(CONDA_VERSION) >= parse_version("4.5.0a0")
 conda_46 = parse_version(CONDA_VERSION) >= parse_version("4.6.0a0")
 conda_47 = parse_version(CONDA_VERSION) >= parse_version("4.7.0a0")
+conda_48 = parse_version(CONDA_VERSION) >= parse_version("4.8.0a0")
 
 if conda_44:
     from conda.exports import display_actions, execute_actions, execute_plan, install_actions
@@ -130,6 +132,12 @@ get_rc_urls = lambda: list(context.channels)
 get_prefix = partial(context_get_prefix, context)
 cc_conda_build = context.conda_build if hasattr(context, 'conda_build') else {}
 
+try:
+    from conda.exports import Channel
+except:
+    from conda.models.channel import Channel
+get_conda_channel = Channel.from_value
+
 # disallow softlinks.  This avoids a lot of dumb issues, at the potential cost of disk space.
 os.environ[str('CONDA_ALLOW_SOFTLINKS')] = str('false')
 reset_context()
