@@ -1,7 +1,7 @@
 '''
 Module to store conda build settings.
 '''
-from __future__ import absolute_import, division, print_function
+from __future__ import absolute_import, division, print_function, unicode_literals
 
 import copy
 from collections import namedtuple
@@ -15,7 +15,7 @@ import time
 from .conda_interface import root_dir, root_writable
 from .conda_interface import binstar_upload
 from .variants import get_default_variant
-from .conda_interface import cc_platform, cc_conda_build, subdir
+from .conda_interface import cc_platform, cc_conda_build, subdir, url_path
 
 from .utils import get_build_folders, rm_rf, get_logger, get_conda_operation_locks
 
@@ -28,12 +28,18 @@ on_win = (sys.platform == 'win32')
 conda_build = "conda-build"
 
 
+# Python2 silliness:
+def python2_fs_encode(strin):
+    return strin.decode(sys.getfilesystemencoding()) if hasattr(strin, 'decode') else strin
+
+
 def _ensure_dir(path):
     # this can fail in parallel operation, depending on timing.  Just try to make the dir,
     #    but don't bail if fail.
-    if not os.path.isdir(path):
+    encpath = python2_fs_encode(path)
+    if not os.path.isdir(encpath):
         try:
-            os.makedirs(path)
+            os.makedirs(encpath)
         except OSError:
             pass
 
@@ -94,6 +100,11 @@ def _get_default_settings():
             # should rendering cut out any skipped metadata?
             Setting('trim_skip', True),
 
+            # Use channeldata.json for run_export information during rendering.
+            # Falls back to downloading packages if False or channeldata does
+            # not exist for the channel.
+            Setting('use_channeldata', False),
+
             # Disable the overlinking test for this package. This test checks that transitive DSOs
             # are not referenced by DSOs in the package being built. When this happens something
             # has gone wrong with:
@@ -109,7 +120,8 @@ def _get_default_settings():
             #    cli/main_build.py that this default will switch in conda-build 4.0.
             Setting('error_overlinking', cc_conda_build.get('error_overlinking',
                                                            'false').lower() == 'true'),
-
+            Setting('enable_static', cc_conda_build.get('enable_static',
+                                                           'false').lower() == 'true'),
             Setting('error_overdepending', cc_conda_build.get('error_overdepending',
                                                               'false').lower() == 'true'),
 
@@ -174,7 +186,14 @@ def _get_default_settings():
             Setting('extra_deps', []),
 
             # customize this so pip doesn't look in places we don't want.  Per-build path by default.
-            Setting('_pip_cache_dir', None)
+            Setting('_pip_cache_dir', None),
+
+            # set up compression algorithm used in new-style packages
+            Setting('compression_tuple', ('.tar.zst', 'zstd', 'zstd:compression-level=22')),
+
+            # this can be set to different values (currently only 2 means anything) to use package formats
+            Setting('conda_pkg_format', cc_conda_build.get('pkg_format', None)),
+
             ]
 
 
@@ -244,7 +263,7 @@ class Config(object):
             self._src_cache_root = os.path.abspath(os.path.normpath(
                 os.path.expanduser(source_cache)))
         if croot:
-            self._croot = os.path.abspath(os.path.normpath(croot))
+            self._croot = os.path.abspath(os.path.normpath(os.path.expanduser(croot)))
         else:
             # set default value (not actually None)
             self._croot = getattr(self, '_croot', None)
@@ -391,7 +410,7 @@ class Config(object):
                 self._croot = join(root_dir, 'conda-bld')
             else:
                 self._croot = abspath(expanduser('~/conda-bld'))
-        return self._croot
+        return python2_fs_encode(self._croot)
 
     @croot.setter
     def croot(self, croot):
@@ -496,7 +515,7 @@ class Config(object):
         return res
 
     def _get_r(self, prefix, platform):
-        if platform.startswith('win'):
+        if platform.startswith('win') or (platform == "noarch" and sys.platform == 'win32'):
             res = join(prefix, 'Scripts', 'R.exe')
             # MRO test:
             if not os.path.exists(res):
@@ -547,7 +566,7 @@ class Config(object):
         _build_id = _build_id.rstrip("/").rstrip("\\")
         assert not os.path.isabs(_build_id), ("build_id should not be an absolute path, "
                                               "to preserve croot during path joins")
-        self._build_id = _build_id
+        self._build_id = python2_fs_encode(_build_id)
 
     @property
     def prefix_length(self):
@@ -604,7 +623,9 @@ class Config(object):
         """The temporary folder where the test environment is created"""
         if on_win or not self.long_test_prefix:
             return self._short_test_prefix
-        return self._long_prefix(self._short_test_prefix)
+        # Reduce the length of the prefix by 2 characters to check if the null
+        # byte padding causes issues
+        return self._long_prefix(self._short_test_prefix)[:-2]
 
     @property
     def build_python(self):
@@ -797,5 +818,22 @@ def get_or_merge_config(config, variant=None, **kwargs):
     return config
 
 
+def get_channel_urls(args):
+    channel_urls = args.get('channel') or args.get('channels') or ()
+    final_channel_urls = []
+
+    for url in channel_urls:
+        # allow people to specify relative or absolute paths to local channels
+        #    These channels still must follow conda rules - they must have the
+        #    appropriate platform-specific subdir (e.g. win-64)
+        if os.path.isdir(url):
+            if not os.path.isabs(url):
+                url = os.path.normpath(os.path.abspath(os.path.join(os.getcwd(), url)))
+            url = url_path(url)
+        final_channel_urls.append(url)
+
+    return final_channel_urls
+
+
 # legacy exports for conda
 croot = Config().croot
