@@ -20,7 +20,7 @@ from .conda_interface import (CondaError, LinkError, LockError, NoPackagesFoundE
 from .conda_interface import display_actions, execute_actions, execute_plan, install_actions
 from .conda_interface import memoized
 from .conda_interface import package_cache, TemporaryDirectory
-from .conda_interface import pkgs_dirs, root_dir, symlink_conda, create_default_packages
+from .conda_interface import pkgs_dirs, root_dir, create_default_packages
 from .conda_interface import reset_context
 
 from conda_build import utils
@@ -243,7 +243,7 @@ def get_hg_build_info(repo):
     return d
 
 
-def get_dict(m, prefix=None, for_env=True, skip_build_id=False, escape_backslash=False):
+def get_dict(m, prefix=None, for_env=True, skip_build_id=False, escape_backslash=False, variant=None):
     if not prefix:
         prefix = m.config.host_prefix
 
@@ -268,7 +268,8 @@ def get_dict(m, prefix=None, for_env=True, skip_build_id=False, escape_backslash
     d.update({feat.upper(): str(int(value)) for feat, value in
               feature_list})
 
-    for k, v in m.config.variant.items():
+    variant = variant or m.config.variant
+    for k, v in variant.items():
         if not for_env or (k.upper() not in d and k.upper() not in LANGUAGES):
             d[k] = v
     return d
@@ -394,12 +395,15 @@ def r_vars(metadata, prefix, escape_backslash):
     if any(r_pkg in deps for r_pkg in R_PACKAGES) or \
             metadata.name(fail_ok=True) in R_PACKAGES:
         r_bin = metadata.config.r_bin(prefix, metadata.config.host_subdir)
+        # set R_USER explicitly to prevent crosstalk with existing R_LIBS_USER packages
+        r_user = join(prefix, 'Libs', 'R')
 
         if utils.on_win and escape_backslash:
             r_bin = r_bin.replace('\\', '\\\\')
 
         vars_.update({
             'R': r_bin,
+            'R_USER': r_user,
         })
     return vars_
 
@@ -591,7 +595,7 @@ def linux_vars(m, get_default, prefix):
     # the GNU triplet is powerpc, not ppc. This matters.
     if build_arch.startswith('ppc'):
         build_arch = build_arch.replace('ppc', 'powerpc')
-    if build_arch.startswith('powerpc'):
+    if build_arch.startswith('powerpc') or build_arch.startswith('aarch64'):
         build_distro = 'cos7'
     else:
         build_distro = 'cos6'
@@ -720,11 +724,10 @@ def get_install_actions(prefix, specs, env, retries=0, subdir=None,
     specs = list(specs)
     if specs:
         specs.extend(create_default_packages)
-    if verbose:
+    if verbose or debug:
         capture = contextlib.contextmanager(lambda: (yield))
-    elif debug:
-        capture = contextlib.contextmanager(lambda: (yield))
-        conda_log_level = logging.DEBUG
+        if debug:
+            conda_log_level = logging.DEBUG
     else:
         capture = utils.capture
     for feature, value in feature_list:
@@ -855,8 +858,7 @@ def create_env(prefix, specs_or_actions, env, config, subdir, clear_cache=True,
                                                     locking=config.locking,
                                                     timeout=config.timeout)
                     utils.trim_empty_keys(actions)
-                    if config.verbose:
-                        display_actions(actions, index)
+                    display_actions(actions, index)
                     if utils.on_win:
                         for k, v in os.environ.items():
                             os.environ[k] = str(v)
@@ -934,15 +936,6 @@ def create_env(prefix, specs_or_actions, env, config, subdir, clear_cache=True,
                     log.error("Failed to create env, max retries exceeded.")
                     raise
 
-    if not is_conda:
-        # Symlinking conda is critical here to make sure that activate scripts are not
-        #    accidentally included in packages.
-        if utils.on_win:
-            shell = "cmd.exe"
-        else:
-            shell = "bash"
-        symlink_conda(prefix, sys.prefix, shell)
-
 
 def get_pkg_dirs_locks(dirs, config):
     return [utils.get_lock(folder, timeout=config.timeout) for folder in dirs]
