@@ -1,10 +1,8 @@
 from __future__ import absolute_import, division, print_function
 
-from collections import defaultdict
-import fnmatch
+from collections import defaultdict, OrderedDict
 from functools import partial
-import glob2
-from glob2 import glob
+from fnmatch import fnmatch, fnmatchcase, filter as fnmatch_filter
 import io
 import locale
 import re
@@ -23,18 +21,18 @@ from conda_build.conda_interface import PY3
 from conda_build.conda_interface import lchmod
 from conda_build.conda_interface import linked_data
 from conda_build.conda_interface import walk_prefix
-from conda_build.conda_interface import pkgs_dirs
 from conda_build.conda_interface import TemporaryDirectory
 from conda_build.conda_interface import md5_file
 
 from conda_build import utils
-from conda_build.os_utils.liefldd import (get_exports_memoized, get_linkages_memoized,
-                                          get_runpaths)
+from conda_build.os_utils.liefldd import (have_lief, get_exports_memoized,
+                                          get_linkages_memoized, get_rpaths_raw,
+                                          get_runpaths_raw, set_rpath)
 from conda_build.os_utils.pyldd import codefile_type
 from conda_build.os_utils.ldd import get_package_obj_files
-from conda_build.index import get_run_exports, get_build_index
+from conda_build.index import get_build_index
 from conda_build.inspect_pkg import which_package
-from conda_build.exceptions import (OverLinkingError, OverDependingError)
+from conda_build.exceptions import (OverLinkingError, OverDependingError, RunPathError)
 
 if sys.platform == 'darwin':
     from conda_build.os_utils import macho
@@ -130,7 +128,7 @@ def remove_easy_install_pth(files, prefix, config, preserve_egg_dir=False):
     absfiles = [os.path.join(prefix, f) for f in files]
     py_ver = '.'.join(config.variant['python'].split('.')[:2])
     sp_dir = utils.get_site_packages(prefix, py_ver)
-    for egg_path in glob(os.path.join(sp_dir, '*-py*.egg')):
+    for egg_path in utils.glob(os.path.join(sp_dir, '*-py*.egg')):
         if os.path.isdir(egg_path):
             if preserve_egg_dir or not any(os.path.join(egg_path, i) in absfiles for i
                     in walk_prefix(egg_path, False, windows_forward_slashes=False)):
@@ -174,6 +172,12 @@ def remove_easy_install_pth(files, prefix, config, preserve_egg_dir=False):
             print('found egg:', egg_path)
             write_pth(egg_path, config=config)
 
+    installer_files = [f for f in absfiles
+                       if f.endswith(".dist-info{}INSTALLER".format(os.path.sep))]
+    for file in installer_files:
+        with open(file, 'w') as f:
+            f.write('conda')
+
     utils.rm_rf(os.path.join(sp_dir, 'easy-install.pth'))
 
 
@@ -225,7 +229,7 @@ def compile_missing_pyc(files, cwd, python_exe, skip_compile_pyc=()):
     skip_compile_pyc_n = [os.path.normpath(skip) for skip in skip_compile_pyc]
     skipped_files = set()
     for skip in skip_compile_pyc_n:
-        skipped_files.update(set(fnmatch.filter(files, skip)))
+        skipped_files.update(set(fnmatch_filter(files, skip)))
     unskipped_files = set(files) - skipped_files
     for fn in unskipped_files:
         # omit files in Library/bin, Scripts, and the root prefix - they are not generally imported
@@ -391,8 +395,8 @@ def osx_ch_link(path, link_dict, host_prefix, build_prefix, files):
 def mk_relative_osx(path, host_prefix, build_prefix, files, rpaths=('lib',)):
     assert sys.platform == 'darwin'
 
-    names = macho.otool(path)
-    s = macho.install_name_change(path,
+    names = macho.otool(path, build_prefix)
+    s = macho.install_name_change(path, build_prefix,
                                   partial(osx_ch_link,
                                           host_prefix=host_prefix,
                                           build_prefix=build_prefix,
@@ -413,10 +417,67 @@ def mk_relative_osx(path, host_prefix, build_prefix, files, rpaths=('lib',)):
     if s:
         # Skip for stub files, which have to use binary_has_prefix_files to be
         # made relocatable.
-        assert_relative_osx(path, host_prefix)
+        assert_relative_osx(path, build_prefix, host_prefix)
 
 
-def mk_relative_linux(f, prefix, rpaths=('lib',)):
+'''
+# Both patchelf and LIEF have bugs in them. Neither can be used on all binaries we have seen.
+# This code tries each and tries to keep count of which worked between the original binary and
+# patchelf-patched, LIEF-patched versions.
+#
+# Please do not delete it until you are sure the bugs in both projects have been fixed.
+#
+
+from subprocess import STDOUT
+
+def check_binary(binary, expected=None):
+    from ctypes import cdll
+    print("trying {}".format(binary))
+    # import pdb; pdb.set_trace()
+    try:
+        txt = check_output([sys.executable, '-c', 'from ctypes import cdll; cdll.LoadLibrary("' + binary + '")'], timeout=2)
+        # mydll = cdll.LoadLibrary(binary)
+    except Exception as e:
+        print(e)
+        return None, None
+    try:
+        txt = check_output(binary, stderr=STDOUT, timeout=0.1)
+    except Exception as e:
+        print(e)
+        txt = e.output
+    if expected is not None:
+        return txt == expected, txt
+    return True, txt
+
+
+worksd = {'original': 0,
+          'LIEF': 0,
+          'patchelf': 0}
+
+
+def check_binary_patchers(elf, prefix, rpath):
+    patchelf = external.find_executable('patchelf', prefix)
+    tmpname_pe = elf+'.patchelf'
+    tmpname_le = elf+'.lief'
+    shutil.copy(elf, tmpname_pe)
+    shutil.copy(elf, tmpname_le)
+    import pdb; pdb.set_trace()
+    works, original = check_binary(elf)
+    if works:
+        worksd['original'] += 1
+        set_rpath(old_matching='*', new_rpath=rpath, file=tmpname_le)
+        works, LIEF = check_binary(tmpname_le, original)
+        call([patchelf, '--force-rpath', '--set-rpath', rpath, tmpname_pe])
+        works, pelf = check_binary(tmpname_pe, original)
+        if original == LIEF and works:
+            worksd['LIEF'] += 1
+        if original == pelf and works:
+            worksd['patchelf'] += 1
+    print('\n' + str(worksd) + '\n')
+'''
+
+
+def mk_relative_linux(f, prefix, rpaths=('lib',), method='LIEF'):
     'Respects the original values and converts abs to $ORIGIN-relative'
 
     elf = os.path.join(prefix, f)
@@ -428,6 +489,11 @@ def mk_relative_linux(f, prefix, rpaths=('lib',)):
     except CalledProcessError:
         print('patchelf: --print-rpath failed for %s\n' % (elf))
         return
+    if have_lief:
+        existing2, _, _ = get_rpaths_raw(elf)
+        if [existing] != existing2:
+            print('ERROR :: get_rpaths_raw()={} and patchelf={} disagree for {} :: '.format(
+                existing2, [existing], elf))
     existing = existing.split(os.pathsep)
     new = []
     for old in existing:
@@ -458,14 +524,20 @@ def mk_relative_linux(f, prefix, rpaths=('lib',)):
             if rpath not in new:
                 new.append(rpath)
     rpath = ':'.join(new)
-    print('patchelf: file: %s\n    setting rpath to: %s' % (elf, rpath))
-    call([patchelf, '--force-rpath', '--set-rpath', rpath, elf])
+
+    # check_binary_patchers(elf, prefix, rpath)
+
+    if method == 'LIEF' or not patchelf:
+        set_rpath(old_matching='*', new_rpath=rpath, file=elf)
+    else:
+        call([patchelf, '--force-rpath', '--set-rpath', rpath, elf])
 
 
-def assert_relative_osx(path, prefix):
+def assert_relative_osx(path, host_prefix, build_prefix):
     for name in macho.get_dylibs(path):
-        if name.startswith(prefix):
-            raise RuntimeError("library at %s appears to have an absolute path embedded" % path)
+        for prefix in (host_prefix, build_prefix):
+            if name.startswith(prefix):
+                raise RuntimeError("library at %s appears to have an absolute path embedded" % path)
 
 
 def determine_package_nature(pkg, prefix, subdir, bldpkgs_dir, output_folder, channel_urls):
@@ -473,9 +545,10 @@ def determine_package_nature(pkg, prefix, subdir, bldpkgs_dir, output_folder, ch
     run_exports = None
     lib_prefix = pkg.name.startswith('lib')
     codefiles = get_package_obj_files(pkg, prefix)
-    dsos = [f for f in codefiles for ext in ('.dylib', '.so', '.dll') if ext in f]
-    # we don't care about the actual run_exports value, just whether or not run_exports are present.  We can use channeldata
-    #    and it'll be a more reliable source (no disk race condition nonsense)
+    # get_package_obj_files already filters by extension and I'm not sure we need two.
+    dsos = [f for f in codefiles for ext in ('.dylib', '.so', '.dll', '.pyd') if ext in f]
+    # we don't care about the actual run_exports value, just whether or not run_exports are present.
+    # We can use channeldata and it'll be a more reliable source (no disk race condition nonsense)
     _, _, channeldata = get_build_index(subdir=subdir,
                                         bldpkgs_dir=bldpkgs_dir,
                                         output_folder=output_folder,
@@ -488,16 +561,6 @@ def determine_package_nature(pkg, prefix, subdir, bldpkgs_dir, output_folder, ch
 
     if channeldata and pkg.name in channeldata['packages']:
         run_exports = channeldata['packages'][pkg.name].get('run_exports', {})
-    else:
-        for pkgs_dir in pkgs_dirs:
-            test_folder = os.path.join(pkgs_dir, pkg.dist_name)
-            test_filename = os.path.join(pkgs_dir, pkg.fn)
-            if os.path.exists(test_folder):
-                run_exports = get_run_exports(test_folder)
-                break
-            elif os.path.isfile(test_filename):
-                run_exports = get_run_exports(test_filename)
-                break
     return (dsos, run_exports, lib_prefix)
 
 
@@ -582,100 +645,126 @@ DEFAULT_MAC_WHITELIST = ['/opt/X11/',
                          '/System/Library/Frameworks/SystemConfiguration.framework/*',
                          '/System/Library/Frameworks/WebKit.framework/*']
 
-DEFAULT_WIN_WHITELIST = ['**/KERNEL32.dll',
-                         '**/ADVAPI32.dll',
+DEFAULT_WIN_WHITELIST = ['**/ADVAPI32.dll',
+                         '**/bcrypt.dll',
+                         '**/COMCTL32.dll',
+                         '**/COMDLG32.dll',
+                         '**/CRYPT32.dll',
+                         '**/dbghelp.dll',
+                         '**/GDI32.dll',
+                         '**/IMM32.dll',
+                         '**/KERNEL32.dll',
+                         '**/NETAPI32.dll',
+                         '**/ole32.dll',
+                         '**/OLEAUT32.dll',
+                         '**/PSAPI.DLL',
                          '**/RPCRT4.dll',
+                         '**/SHELL32.dll',
+                         '**/USER32.dll',
+                         '**/USERENV.dll',
+                         '**/WINHTTP.dll',
+                         '**/WS2_32.dll',
                          '**/ntdll.dll',
                          '**/msvcrt.dll',
                          '**/api-ms-win*.dll']
 
 
-def _collect_needed_dsos(sysroots, files, run_prefix, sysroot_substitution, build_prefix, build_prefix_substitution):
+def _collect_needed_dsos(sysroots_files, files, run_prefix, sysroot_substitution, build_prefix, build_prefix_substitution):
     all_needed_dsos = set()
     needed_dsos_for_file = dict()
-    sysroot = sysroots[0] if sysroots else ''
+    sysroot = ''
+    if sysroots_files:
+        sysroot = list(sysroots_files.keys())[0]
     for f in files:
         path = os.path.join(run_prefix, f)
         if not codefile_type(path):
             continue
+        build_prefix = build_prefix.replace(os.sep, '/')
+        run_prefix = run_prefix.replace(os.sep, '/')
         needed = get_linkages_memoized(path, resolve_filenames=True, recurse=False,
-                                       sysroot=sysroot,
-                                       envroot=run_prefix)
+                                    sysroot=sysroot,
+                                    envroot=run_prefix)
         if sysroot:
             needed = [n.replace(sysroot, sysroot_substitution) if n.startswith(sysroot)
-                      else n for n in needed]
-        needed = [n.replace(build_prefix, build_prefix_substitution) if n.startswith(build_prefix)
-                  else n for n in needed]
-        needed = [os.path.relpath(n, run_prefix) if n.startswith(run_prefix)
-                  else n for n in needed]
+                    else n for n in needed]
+        # We do not want to do this substitution when merging build and host prefixes.
+        if build_prefix != run_prefix:
+            needed = [n.replace(build_prefix, build_prefix_substitution) if n.startswith(build_prefix)
+                    else n for n in needed]
+        needed = [os.path.relpath(n, run_prefix).replace(os.sep, '/') if n.startswith(run_prefix)
+                else n for n in needed]
         needed_dsos_for_file[f] = needed
         all_needed_dsos = all_needed_dsos.union(needed)
         all_needed_dsos.add(f)
     return all_needed_dsos, needed_dsos_for_file
 
 
-def _map_file_to_package(files, run_prefix, build_prefix, all_needed_dsos, pkg_vendored_dist, ignore_list_syms, sysroot_substitution):
+def _map_file_to_package(files, run_prefix, build_prefix, all_needed_dsos, pkg_vendored_dist, ignore_list_syms, sysroot_substitution, enable_static):
     # Form a mapping of file => package
     prefix_owners = {}
     contains_dsos = {}
     contains_static_libs = {}
     # Used for both dsos and static_libs
     all_lib_exports = {}
-    for prefix in (run_prefix, build_prefix):
-        for subdir2, _, filez in os.walk(prefix):
-            for file in filez:
-                fp = os.path.join(subdir2, file)
-                dynamic_lib = any(glob2.fnmatch.fnmatch(fp, ext) for ext in ('*.so*', '*.dylib*', '*.dll')) and \
-                              codefile_type(fp, skip_symlinks=False) is not None
-                static_lib = any(glob2.fnmatch.fnmatch(fp, ext) for ext in ('*.a', '*.lib'))
-                # Looking at all the files is very slow.
-                if not dynamic_lib and not static_lib:
-                    continue
-                rp = os.path.relpath(fp, prefix)
-                if dynamic_lib and rp not in all_needed_dsos:
-                    continue
-                if rp in all_lib_exports:
-                    continue
-                owners = prefix_owners[rp] if rp in prefix_owners else []
-                # Self-vendoring, not such a big deal but may as well report it?
-                if not len(owners):
-                    if rp in files:
-                        owners.append(pkg_vendored_dist)
-                new_pkgs = list(which_package(rp, prefix))
-                # Cannot filter here as this means the DSO (eg libomp.dylib) will not be found in any package
-                # [owners.append(new_pkg) for new_pkg in new_pkgs if new_pkg not in owners
-                #  and not any([glob2.fnmatch.fnmatch(new_pkg.name, i) for i in ignore_for_statics])]
-                for new_pkg in new_pkgs:
-                    if new_pkg not in owners:
-                        owners.append(new_pkg)
-                prefix_owners[rp] = owners
-                if len(prefix_owners[rp]):
-                    exports = set(e for e in get_exports_memoized(fp) if not
-                                  any(glob2.fnmatch.fnmatch(e, pattern) for pattern in ignore_list_syms))
-                    all_lib_exports[rp] = exports
-                    # Check codefile_type to filter out linker scripts.
-                    if dynamic_lib:
-                        contains_dsos[prefix_owners[rp][0]] = True
-                    elif static_lib:
-                        if sysroot_substitution in fp:
-                            if (prefix_owners[rp][0].name.startswith('gcc_impl_linux') or
-                               prefix_owners[rp][0].name == 'llvm'):
-                                continue
-                            print("sysroot in {}, owner is {}".format(fp, prefix_owners[rp][0]))
-                        contains_static_libs[prefix_owners[rp][0]] = True
+    if all_needed_dsos:
+        for prefix in (run_prefix, build_prefix):
+            for subdir2, _, filez in os.walk(prefix):
+                for file in filez:
+                    fp = os.path.join(subdir2, file)
+                    dynamic_lib = any(fnmatch(fp, ext) for ext in ('*.so.*', '*.dylib.*', '*.dll')) and \
+                                codefile_type(fp, skip_symlinks=False) is not None
+                    static_lib = any(fnmatch(fp, ext) for ext in ('*.a', '*.lib'))
+                    # Looking at all the files is very slow.
+                    if not dynamic_lib and not static_lib:
+                        continue
+                    rp = os.path.relpath(fp, prefix)
+                    if dynamic_lib and rp not in all_needed_dsos:
+                        continue
+                    if rp in all_lib_exports:
+                        continue
+                    owners = prefix_owners[rp] if rp in prefix_owners else []
+                    # Self-vendoring, not such a big deal but may as well report it?
+                    if not len(owners):
+                        if rp in files:
+                            owners.append(pkg_vendored_dist)
+                    new_pkgs = list(which_package(rp, prefix))
+                    # Cannot filter here as this means the DSO (eg libomp.dylib) will not be found in any package
+                    # [owners.append(new_pkg) for new_pkg in new_pkgs if new_pkg not in owners
+                    #  and not any([fnmatch(new_pkg.name, i) for i in ignore_for_statics])]
+                    for new_pkg in new_pkgs:
+                        if new_pkg not in owners:
+                            owners.append(new_pkg)
+                    prefix_owners[rp] = owners
+                    if len(prefix_owners[rp]):
+                        exports = set(e for e in get_exports_memoized(fp, enable_static=enable_static) if not
+                                    any(fnmatch(e, pattern) for pattern in ignore_list_syms))
+                        all_lib_exports[rp] = exports
+                        # Check codefile_type to filter out linker scripts.
+                        if dynamic_lib:
+                            contains_dsos[prefix_owners[rp][0]] = True
+                        elif static_lib:
+                            if sysroot_substitution in fp:
+                                if (prefix_owners[rp][0].name.startswith('gcc_impl_linux') or
+                                prefix_owners[rp][0].name == 'llvm'):
+                                    continue
+                                print("sysroot in {}, owner is {}".format(fp, prefix_owners[rp][0]))
+                            contains_static_libs[prefix_owners[rp][0]] = True
     return prefix_owners, contains_dsos, contains_static_libs, all_lib_exports
 
 
 def _get_fake_pkg_dist(pkg_name, pkg_version, build_str, build_number):
     pkg_vendoring_name = pkg_name
-    pkg_vendoring_version = pkg_version
+    pkg_vendoring_version = str(pkg_version)
     pkg_vendoring_build_str = build_str
     pkg_vendoring_build_number = build_number
+    pkg_vendoring_key = '-'.join([pkg_vendoring_name,
+                                  pkg_vendoring_version,
+                                  pkg_vendoring_build_str])
 
     return FakeDist(pkg_vendoring_name,
-                                 pkg_vendoring_version,
-                                 pkg_vendoring_build_number,
-                                 pkg_vendoring_build_str)
+                    pkg_vendoring_version,
+                    pkg_vendoring_build_number,
+                    pkg_vendoring_build_str), pkg_vendoring_key
 
 
 def _print_msg(errors, text, verbose):
@@ -685,38 +774,38 @@ def _print_msg(errors, text, verbose):
         print(text)
 
 
-def _lookup_in_system_whitelists(errors, whitelist, needed_dso, sysroots, msg_prelude, info_prelude,
+def _lookup_in_system_whitelists(errors, whitelist, needed_dso, sysroots_files, msg_prelude, info_prelude,
                                  sysroot_prefix, sysroot_substitution, verbose):
-    # A system or ignored dependency. We should be able to find it in one of the CDT o
-    # compiler packages on linux or at in a sysroot folder on other OSes. These usually
+    # A system or ignored dependency. We should be able to find it in one of the CDT or
+    # compiler packages on linux or in a sysroot folder on other OSes. These usually
     # start with '$RPATH/' which indicates pyldd did not find them, so remove that now.
     if needed_dso.startswith(sysroot_substitution):
-        replacements = [sysroot_substitution] + sysroots
+        replacements = [sysroot_substitution] + [sysroot for sysroot, _ in sysroots_files.items()]
     else:
         replacements = [needed_dso]
     in_whitelist = False
+    # It takes a very long time to glob in C:/Windows so cache it.
     for replacement in replacements:
         needed_dso_w = needed_dso.replace(sysroot_substitution, replacement)
-        in_whitelist = any([glob2.fnmatch.fnmatch(needed_dso_w, w) for w in whitelist])
+        in_whitelist = any([fnmatch(needed_dso_w, w) for w in whitelist])
         if in_whitelist:
             n_dso_p = "Needed DSO {}".format(needed_dso_w)
             _print_msg(errors, '{}: {} found in the whitelist'.
-                        format(info_prelude, n_dso_p), verbose=verbose)
+                       format(info_prelude, n_dso_p), verbose=verbose)
             break
-    if not in_whitelist and len(sysroots):
+    if not in_whitelist and len(sysroots_files):
         # Check if we have a CDT package.
         dso_fname = os.path.basename(needed_dso)
         sysroot_files = []
-        dirs_to_glob = []  # Optimization, ideally we'll not glob at all as it's slooow.
-        for sysroot in sysroots:
+        for sysroot, files in sysroots_files.items():
             sysroot_os = sysroot.replace('/', os.sep)
+            wild = os.path.join('**', dso_fname)
             if needed_dso.startswith(sysroot_substitution):
                 # Do we want to do this replace?
                 sysroot_files.append(needed_dso.replace(sysroot_substitution, sysroot_os))
             else:
-                dirs_to_glob.extend((os.path.join(sysroot_os, '**', dso_fname)))
-        for dir_to_glob in dirs_to_glob:
-            sysroot_files.extend(glob(dir_to_glob))
+                found = [file for file in files if fnmatchcase(file, wild)]
+                sysroot_files.extend(found)
         if len(sysroot_files):
             # Removing sysroot_prefix is only *really* for Linux, though we could
             # use CONDA_BUILD_SYSROOT for macOS. We should figure out what to do about
@@ -749,9 +838,6 @@ def _lookup_in_system_whitelists(errors, whitelist, needed_dso, sysroots, msg_pr
 def _lookup_in_prefix_packages(errors, needed_dso, files, run_prefix, whitelist, info_prelude, msg_prelude,
                                warn_prelude, verbose, requirements_run, lib_packages, lib_packages_used):
     in_prefix_dso = needed_dso
-    if in_prefix_dso == '/':
-        print('debug')
-    # os.path.normpath(needed_dso.replace(run_prefix + os.sep, ''))
     n_dso_p = "Needed DSO {}".format(in_prefix_dso)
     and_also = " (and also in this package)" if in_prefix_dso in files else ""
     pkgs = list(which_package(in_prefix_dso, run_prefix))
@@ -760,7 +846,7 @@ def _lookup_in_prefix_packages(errors, needed_dso, files, run_prefix, whitelist,
     for pkg in in_pkgs_in_run_reqs:
         if pkg in lib_packages:
             lib_packages_used.add(pkg)
-    in_whitelist = any([glob2.fnmatch.fnmatch(in_prefix_dso, w) for w in whitelist])
+    in_whitelist = any([fnmatch(in_prefix_dso, w) for w in whitelist])
     if len(in_pkgs_in_run_reqs) == 1:
         _print_msg(errors, '{}: {} found in {}{}'.format(info_prelude,
                                                         n_dso_p,
@@ -806,25 +892,25 @@ def _show_linking_messages(files, errors, needed_dsos_for_file, build_prefix, ru
         msg_prelude = err_prelude if error_overlinking else warn_prelude
 
         try:
-            runpaths = get_runpaths(path)
+            runpaths, _, _ = get_runpaths_raw(path)
         except:
-            _print_msg(errors, '{}: pyldd.py failed to process'.format(warn_prelude))
+            _print_msg(errors, '{}: pyldd.py failed to process'.format(warn_prelude),
+                       verbose=verbose)
             continue
         if runpaths and not (runpath_whitelist or
-                             any(fnmatch.fnmatch(f, w) for w in runpath_whitelist)):
+                             any(fnmatch(f, w) for w in runpath_whitelist)):
             _print_msg(errors, '{}: runpaths {} found in {}'.format(msg_prelude,
                                                                     runpaths,
                                                                     path), verbose=verbose)
         needed = needed_dsos_for_file[f]
-        # imps = get_imports_memoized(path, None)
         for needed_dso in needed:
             needed_dso = needed_dso.replace('/', os.sep)
             if not needed_dso.startswith(os.sep) and not needed_dso.startswith('$'):
                 _lookup_in_prefix_packages(errors, needed_dso, files, run_prefix, whitelist, info_prelude, msg_prelude,
                                warn_prelude, verbose, requirements_run, lib_packages, lib_packages_used)
-            elif needed_dso.startswith(build_prefix):
+            elif needed_dso.startswith('$PATH'):
                 _print_msg(errors, "{}: {} found in build prefix; should never happen".format(
-                          err_prelude, needed_dso), verbose=verbose)
+                           err_prelude, needed_dso), verbose=verbose)
             else:
                 _lookup_in_system_whitelists(errors, whitelist, needed_dso, sysroots, msg_prelude,
                                              info_prelude, sysroot_prefix, sysroot_substitution, verbose)
@@ -836,12 +922,23 @@ def check_overlinking_impl(pkg_name, pkg_version, build_str, build_number, subdi
                            run_prefix, build_prefix,
                            missing_dso_whitelist, runpath_whitelist,
                            error_overlinking, error_overdepending, verbose,
-                           exception_on_error, files, bldpkgs_dirs, output_folder, channel_urls):
+                           exception_on_error, files, bldpkgs_dirs, output_folder, channel_urls,
+                           enable_static=False):
     verbose = True
     errors = []
 
-    sysroot_substitution = '$SYSROOT/'
-    build_prefix_substitution = '$PATH/'
+    files_to_inspect = []
+    for f in files:
+        path = os.path.join(run_prefix, f)
+        filetype = codefile_type(path)
+        if filetype and filetype in filetypes_for_platform[subdir.split('-')[0]]:
+            files_to_inspect.append(f)
+
+    if not files_to_inspect:
+        return dict()
+
+    sysroot_substitution = '$SYSROOT'
+    build_prefix_substitution = '$PATH'
     # Used to detect overlinking (finally)
     requirements_run = [req.split(' ')[0] for req in requirements_run]
     packages = dists_from_names(requirements_run, run_prefix)
@@ -860,49 +957,74 @@ def check_overlinking_impl(pkg_name, pkg_version, build_str, build_number, subdi
     else:
         lib_packages_used = set()
 
-    pkg_vendored_dist = _get_fake_pkg_dist(pkg_name, pkg_version, build_str, build_number)
+    pkg_vendored_dist, pkg_vendoring_key = _get_fake_pkg_dist(pkg_name, pkg_version, build_str, build_number)
 
     ignore_list_syms = ['main', '_main', '*get_pc_thunk*', '___clang_call_terminate', '_timeout']
     # ignore_for_statics = ['gcc_impl_linux*', 'compiler-rt*', 'llvm-openmp*', 'gfortran_osx*']
     # sysroots and whitelists are similar, but the subtle distinctions are important.
     sysroot_prefix = build_prefix
-    sysroots = [sysroot + os.sep for sysroot in glob(os.path.join(sysroot_prefix, '**', 'sysroot'))]
+    sysroots = [sysroot + os.sep for sysroot in utils.glob(os.path.join(sysroot_prefix, '**', 'sysroot'))]
     whitelist = []
+    vendoring_record = dict()
+    # When build_is_host is True we perform file existence checks for files in the sysroot (e.g. C:\Windows)
+    # When build_is_host is False we must skip things that match the whitelist from the prefix_owners (we could
+    #   create some packages for the Windows System DLLs as an alternative?)
+    build_is_host = False
     if not len(sysroots):
         if subdir == 'osx-64':
-            sysroots = ['/usr/lib/', '/opt/X11/', '/System/Library/Frameworks/']
+            # This is a bit confused! A sysroot shouldn't contain /usr/lib (it's the bit before that)
+            # what we are really specifying here are subtrees of sysroots to search in and it may be
+            # better to store each element of this as a tuple with a string and a nested tuple, e.g.
+            # [('/', ('/usr/lib', '/opt/X11', '/System/Library/Frameworks'))]
+            # Here we mean that we have a sysroot at '/' (could be a tokenized value like '$SYSROOT'?)
+            # .. and in that sysroot there are 3 suddirs in which we may search for DSOs.
+            sysroots = ['/usr/lib', '/opt/X11', '/System/Library/Frameworks']
             whitelist = DEFAULT_MAC_WHITELIST
+            build_is_host = True if sys.platform == 'darwin' else False
         elif subdir.startswith('win'):
             sysroots = ['C:/Windows']
             whitelist = DEFAULT_WIN_WHITELIST
+            build_is_host = True if sys.platform == 'win-32' else False
 
-    # LIEF is very slow at decoding some DSOs, so we only let it look at ones that we link to (and ones we
-    # have built).
-    all_needed_dsos, needed_dsos_for_file = _collect_needed_dsos(sysroots, files, run_prefix, sysroot_substitution,
+    # Sort the sysroots by the number of files in them so things can assume that
+    # the first sysroot is more important than others.
+    sysroots_files = dict()
+    for sysroot in sysroots:
+        from conda_build.utils import prefix_files
+        sysroots_files[sysroot] = prefix_files(sysroot)
+    sysroots_files = OrderedDict(sorted(sysroots_files.items(), key=lambda x: -len(x[1])))
+
+    all_needed_dsos, needed_dsos_for_file = _collect_needed_dsos(sysroots_files, files, run_prefix,
+                                                                 sysroot_substitution,
                                                                  build_prefix, build_prefix_substitution)
 
     prefix_owners, _, _, all_lib_exports = _map_file_to_package(
-        files, run_prefix, build_prefix, all_needed_dsos, pkg_vendored_dist, ignore_list_syms, sysroot_substitution)
+        files, run_prefix, build_prefix, all_needed_dsos, pkg_vendored_dist, ignore_list_syms,
+        sysroot_substitution, enable_static)
 
-    for f in files:
-        path = os.path.join(run_prefix, f)
-        filetype = codefile_type(path)
-        if not filetype or filetype not in filetypes_for_platform[subdir.split('-')[0]]:
-            continue
+    for f in files_to_inspect:
         needed = needed_dsos_for_file[f]
         for needed_dso in needed:
             if (error_overlinking and
                not needed_dso.startswith('/') and
                not needed_dso.startswith(sysroot_substitution) and
                not needed_dso.startswith(build_prefix_substitution) and
-               needed_dso not in prefix_owners):
-                print("  ERROR :: {} not in prefix_owners".format(needed_dso))
-                sys.exit(1)
+               needed_dso not in prefix_owners and
+               needed_dso not in files):
+                in_whitelist = False
+                if not build_is_host:
+                    in_whitelist = any([fnmatch(needed_dso, w) for w in whitelist])
+                if not in_whitelist:
+                    print("  ERROR :: {} not in prefix_owners".format(needed_dso))
+
+    # Should the whitelist be expanded before the 'not in prefix_owners' check?
+    # i.e. Do we want to be able to use the whitelist to allow missing files in general? If so move this up to
+    # the line before 'for needed_dso in needed'
+    whitelist += missing_dso_whitelist or []
 
-    whitelist += missing_dso_whitelist
     _show_linking_messages(files, errors, needed_dsos_for_file, build_prefix, run_prefix, pkg_name,
                            error_overlinking, runpath_whitelist, verbose, requirements_run, lib_packages,
-                           lib_packages_used, whitelist, sysroots, sysroot_prefix, sysroot_substitution, subdir)
+                           lib_packages_used, whitelist, sysroots_files, sysroot_prefix, sysroot_substitution, subdir)
 
     if lib_packages_used != lib_packages:
         info_prelude = "   INFO ({})".format(pkg_name)
@@ -916,12 +1038,15 @@ def check_overlinking_impl(pkg_name, pkg_version, build_str, build_number, subdi
             else:
                 msg_prelude = warn_prelude
             _print_msg(errors, "{}: {} package {} in requirements/run but it is not used "
-                              "(i.e. it is overdepending or perhaps statically linked? "
-                              "If that is what you want then add it to `build/ignore_run_exports`)"
-                              .format(msg_prelude, package_nature[lib], lib), verbose=verbose)
+                               "(i.e. it is overdepending or perhaps statically linked? "
+                               "If that is what you want then add it to `build/ignore_run_exports`)"
+                               .format(msg_prelude, package_nature[lib], lib), verbose=verbose)
     if len(errors):
         if exception_on_error:
-            overlinking_errors = [error for error in errors if "overlinking" in error]
+            runpaths_errors = [error for error in errors if re.match(r".*runpaths.*found in.*", error)]
+            if len(runpaths_errors):
+                raise RunPathError(runpaths_errors)
+            overlinking_errors = [error for error in errors if re.match(r".*(overlinking|not found in|did not find).*", error)]
             if len(overlinking_errors):
                 raise OverLinkingError(overlinking_errors)
             overdepending_errors = [error for error in errors if "overdepending" in error]
@@ -930,6 +1055,12 @@ def check_overlinking_impl(pkg_name, pkg_version, build_str, build_number, subdi
         else:
             sys.exit(1)
 
+    if pkg_vendoring_key in vendoring_record:
+        imports = vendoring_record[pkg_vendoring_key]
+        return imports
+    else:
+        return dict()
+
 
 def check_overlinking(m, files):
     return check_overlinking_impl(m.get_value('package/name'),
@@ -952,7 +1083,8 @@ def check_overlinking(m, files):
                                   files,
                                   m.config.bldpkgs_dir,
                                   m.config.output_folder,
-                                  m.config.channel_urls)
+                                  m.config.channel_urls,
+                                  m.config.enable_static)
 
 
 def post_process_shared_lib(m, f, files):
@@ -962,7 +1094,8 @@ def post_process_shared_lib(m, f, files):
         return
     rpaths = m.get_value('build/rpaths', ['lib'])
     if sys.platform.startswith('linux') and codefile_t == 'elffile':
-        mk_relative_linux(f, m.config.host_prefix, rpaths=rpaths)
+        mk_relative_linux(f, m.config.host_prefix, rpaths=rpaths,
+                          method=m.get_value('build/rpaths_patcher', 'patchelf'))
     elif sys.platform == 'darwin' and codefile_t == 'machofile':
         mk_relative_osx(path, m.config.host_prefix, m.config.build_prefix, files=files, rpaths=rpaths)
 
@@ -994,8 +1127,9 @@ def fix_permissions(files, prefix):
 def post_build(m, files, build_python):
     print('number of files:', len(files))
 
+    host_prefix = m.config.host_prefix
     for f in files:
-        make_hardlink_copy(f, m.config.host_prefix)
+        make_hardlink_copy(f, host_prefix)
 
     if not m.config.target_subdir.startswith('win'):
         binary_relocation = m.binary_relocation()
@@ -1003,19 +1137,17 @@ def post_build(m, files, build_python):
             print("Skipping binary relocation logic")
         osx_is_app = (m.config.target_subdir == 'osx-64' and
                       bool(m.get_value('build/osx_is_app', False)))
-        check_symlinks(files, m.config.host_prefix, m.config.croot)
-        prefix_files = utils.prefix_files(m.config.host_prefix)
+        check_symlinks(files, host_prefix, m.config.croot)
+        prefix_files = utils.prefix_files(host_prefix)
 
         for f in files:
             if f.startswith('bin/'):
-                fix_shebang(f, prefix=m.config.host_prefix, build_python=build_python,
+                fix_shebang(f, prefix=host_prefix, build_python=build_python,
                             osx_is_app=osx_is_app)
             if binary_relocation is True or (isinstance(binary_relocation, list) and
                                              f in binary_relocation):
                 post_process_shared_lib(m, f, prefix_files)
-    # disable overlinking check on win right now, until Ray has time for it.
-    if not utils.on_win:
-        check_overlinking(m, files)
+    check_overlinking(m, files)
 
 
 def check_symlinks(files, prefix, croot):
