@@ -61,8 +61,19 @@ def bldpkg_path(m):
     '''
     subdir = 'noarch' if m.noarch or m.noarch_python else m.config.host_subdir
 
-    if not hasattr(m, 'type') or m.type == "conda":
+    if not hasattr(m, 'type'):
+        if m.config.conda_pkg_format == "2":
+            pkg_type = "conda_v2"
+        else:
+            pkg_type = "conda"
+    else:
+        pkg_type = m.type
+
+    # the default case will switch over to conda_v2 at some point
+    if pkg_type == "conda":
         path = os.path.join(m.config.output_folder, subdir, '%s%s' % (m.dist(), CONDA_TARBALL_EXTENSIONS[0]))
+    elif pkg_type == "conda_v2":
+        path = os.path.join(m.config.output_folder, subdir, '%s%s' % (m.dist(), '.conda'))
     else:
         path = '{} file for {} in: {}'.format(m.type, m.name(), os.path.join(m.config.output_folder, subdir))
     return path
@@ -73,7 +84,7 @@ def actions_to_pins(actions):
     if conda_43:
         spec_name = lambda x: x.dist_name
     else:
-        spec_name = lambda x: x
+        spec_name = lambda x: str(x)
     if 'LINK' in actions:
         specs = [' '.join(spec_name(spec).split()[0].rsplit('-', 2)) for spec in actions['LINK']]
     return specs
@@ -242,7 +253,7 @@ def _read_specs_from_package(pkg_loc, pkg_dist):
         elif os.path.isfile(downstream_file + '.json'):
             with open(downstream_file + '.json') as f:
                 specs = json.load(f)
-    if not specs and os.path.isfile(pkg_loc):
+    if not specs and pkg_loc and os.path.isfile(pkg_loc):
         # switching to json for consistency in conda-build 4
         specs_yaml = utils.package_has_file(pkg_loc, 'info/run_exports.yaml')
         specs_json = utils.package_has_file(pkg_loc, 'info/run_exports.json')
@@ -276,10 +287,11 @@ def execute_download_actions(m, actions, env, package_subset=None, require_files
                                   timeout=m.config.timeout)
 
     # this should be just downloading packages.  We don't need to extract them -
-    #    we read contents directly
+
+    download_actions = {k: v for k, v in actions.items() if k in ('FETCH', 'EXTRACT', 'PREFIX')}
     if 'FETCH' in actions or 'EXTRACT' in actions:
         # this is to force the download
-        execute_actions(actions, index, verbose=m.config.debug)
+        execute_actions(download_actions, index, verbose=m.config.debug)
 
     pkg_files = {}
 
@@ -340,16 +352,23 @@ def get_upstream_pins(m, actions, env):
     linked_packages = actions.get('LINK', [])
     linked_packages = [pkg for pkg in linked_packages if pkg.name in explicit_specs]
 
-    pkg_locs_and_dists = execute_download_actions(m, actions, env=env,
-                                                  package_subset=linked_packages)
-
     ignore_list = utils.ensure_list(m.get_value('build/ignore_run_exports'))
-
     additional_specs = {}
-    for (loc, dist) in pkg_locs_and_dists.values():
-        specs = _read_specs_from_package(loc, dist)
-        additional_specs = utils.merge_dicts_of_lists(additional_specs,
-                                                      _filter_run_exports(specs, ignore_list))
+    for pkg in linked_packages:
+        run_exports = None
+        if m.config.use_channeldata:
+            channeldata = utils.download_channeldata(pkg.channel)
+            # only use channeldata if requested, channeldata exists and contains
+            # a packages key, otherwise use run_exports from the packages themselves
+            if 'packages' in channeldata:
+                pkg_data = channeldata['packages'].get(pkg.name, {})
+                run_exports = pkg_data.get('run_exports', {}).get(pkg.version, {})
+        if run_exports is None:
+            loc, dist = execute_download_actions(m, actions, env=env, package_subset=pkg)[pkg]
+            run_exports = _read_specs_from_package(loc, dist)
+        specs = _filter_run_exports(run_exports, ignore_list)
+        if specs:
+            additional_specs = utils.merge_dicts_of_lists(additional_specs, specs)
     return additional_specs
 
 
@@ -459,10 +478,8 @@ def finalize_metadata(m, parent_metadata=None, permit_unsatisfiable_variants=Fal
     if not parent_metadata:
         parent_metadata = m
     if m.skip():
-        rendered_metadata = m.copy()
-        rendered_metadata.final = True
+        m.final = True
     else:
-
         exclude_pattern = None
         excludes = set(m.config.variant.get('ignore_version', []))
 
@@ -483,14 +500,15 @@ def finalize_metadata(m, parent_metadata=None, permit_unsatisfiable_variants=Fal
         # extract the topmost section where variables are defined, and put it on top of the
         #     requirements for a particular output
         # Re-parse the output from the original recipe, so that we re-consider any jinja2 stuff
-        parent_metadata = parent_metadata.copy()
-        parent_metadata.config.variant = m.config.variant
-        output = parent_metadata.get_rendered_output(m.name())
+        output = parent_metadata.get_rendered_output(m.name(), variant=m.config.variant)
 
+        is_top_level = True
         if output:
             if 'package' in output or 'name' not in output:
                 # it's just a top-level recipe
                 output = {'name': m.name()}
+            else:
+                is_top_level = False
 
             if not parent_recipe or parent_recipe['name'] == m.name():
                 combine_top_level_metadata_with_output(m, output)
@@ -511,15 +529,6 @@ def finalize_metadata(m, parent_metadata=None, permit_unsatisfiable_variants=Fal
         #     to the metadata.
         requirements = m.meta.get('requirements', {})
 
-        # this is hacky, but it gets the jinja2 things like pin_compatible from the rendered output
-        # rerendered_output = parent_metadata.get_output_metadata(m.get_rendered_output(m.name()))
-        # run_reqs = utils.expand_reqs(rerendered_output.meta.get('requirements', {}))
-        # run_reqs = run_reqs.get('run', [])
-        # if run_reqs:
-        #     requirements['run'] = run_reqs
-        # m.meta['requirements'] = requirements
-        # m.meta['build'] = rerendered_output.meta.get('build', {})
-
         # here's where we pin run dependencies to their build time versions.  This happens based
         #     on the keys in the 'pin_run_as_build' key in the variant, which is a list of package
         #     names to have this behavior.
@@ -606,6 +615,8 @@ def finalize_metadata(m, parent_metadata=None, permit_unsatisfiable_variants=Fal
                 log.warn("Host: {}".format(host_unsat))
         else:
             m.final = True
+    if is_top_level:
+        parent_metadata = m
     return m
 
 
@@ -789,7 +800,6 @@ def render_recipe(recipe_path, config, no_download_source=False, variants=None,
     #   folder until rendering is complete, because package names can have variant jinja2 in them.
     if m.needs_source_for_render and not m.source_provided:
         try_download(m, no_download_source=no_download_source)
-
     if m.final:
         if not hasattr(m.config, 'variants') or not m.config.variant:
             m.config.ignore_system_variants = True
