@@ -67,7 +67,7 @@ about:
   summary: {summary}
 
 # See
-# http://docs.continuum.io/conda/build.html for
+# https://docs.conda.io/projects/conda-build for
 # more information about meta.yaml
 """
 
@@ -95,7 +95,7 @@ fi
 # Add more build steps here, if they are necessary.
 
 # See
-# http://docs.continuum.io/conda/build.html
+# https://docs.conda.io/projects/conda-build
 # for a list of environment variables that are set during the build process.
 """
 
@@ -127,7 +127,7 @@ IF exist Build.PL (
 :: Add more build steps here, if they are necessary.
 
 :: See
-:: http://docs.continuum.io/conda/build.html
+:: https://docs.conda.io/projects/conda-build
 :: for a list of environment variables that are set during the build process.
 """
 
@@ -207,6 +207,7 @@ def package_exists(package_name):
         cmd = ['cpan', '-D', package_name]
         if on_win:
             cmd.insert(0, '/c')
+            cmd.insert(0, '/d')
             cmd.insert(0, 'cmd.exe')
         check_call_env(cmd)
         in_repo = True
