@@ -72,8 +72,8 @@ PYPI_META_HEADER = """{{% set name = "{packagename}" %}}
 # etc. is determined by EXPECTED_SECTION_ORDER.
 PYPI_META_STATIC = {
     'package': OrderedDict([
-        ('name', '"{{ name|lower }}"'),
-        ('version', '"{{ version }}"'),
+        ('name', '{{ name|lower }}'),
+        ('version', '{{ version }}'),
     ]),
     'source': OrderedDict([
         ('url', '/packages/source/{{ name[0] }}/{{ name }}/{{ name }}-{{ version }}.tar.gz'),  # NOQA
@@ -173,7 +173,10 @@ def _print_dict(d, order=None, level=0, indent=2):
                 if isinstance(_v, string_types) or not hasattr(_v, "__iter__"):
                     rendered_recipe += __print_with_indent(_k, suffix=':', level=level + indent,
                                                           newline=False)
-                    rendered_recipe += ' ' + str(_v) + '\n'
+                    if isinstance(_v, string_types):
+                        rendered_recipe += ' "' + _v + '"\n'
+                    else:
+                        rendered_recipe += ' ' + str(_v) + '\n'
                 elif hasattr(_v, 'keys'):
                     rendered_recipe += _print_dict(_v, sorted(list(_v.keys())))
                 # assume that it's a list if it exists at all
@@ -339,9 +342,9 @@ def skeletonize(packages, output_dir=".", version=None, recursive=False,
             if noarch_python:
                 ordered_recipe['build']['noarch'] = 'python'
 
-            ordered_recipe['build']['script'] = ('"{{ PYTHON }} -m pip install . --no-deps '
+            ordered_recipe['build']['script'] = ('{{ PYTHON }} -m pip install . --no-deps '
                                                  '--ignore-installed -vv ' +
-                                                 ' '.join(setup_options) + '"')
+                                                 ' '.join(setup_options))
 
             # Always require python as a dependency.  Pip is because we use pip for
             #    the install line.
@@ -699,62 +702,13 @@ def get_package_metadata(package, d, data, output_dir, python_version, all_extra
                           setup_options=setup_options,
                           config=config)
 
-    setuptools_run = False
-
-    # Look at the entry_points and construct console_script and
-    #  gui_scripts entry_points for conda
-    entry_points = pkginfo.get('entry_points', [])
-    if entry_points:
-        if isinstance(entry_points, str):
-            # makes sure it is left-shifted
-            newstr = "\n".join(x.strip()
-                                for x in entry_points.splitlines())
-            _config = configparser.ConfigParser()
-            entry_points = {}
-            try:
-                _config.readfp(StringIO(newstr))
-            except Exception as err:
-                print("WARNING: entry-points not understood: ",
-                        err)
-                print("The string was", newstr)
-                entry_points = pkginfo['entry_points']
-            else:
-                setuptools_run = True
-                for section in _config.sections():
-                    if section in ['console_scripts', 'gui_scripts']:
-                        value = ['%s=%s' % (option, _config.get(section, option))
-                                    for option in _config.options(section)]
-                        entry_points[section] = value
-        if not isinstance(entry_points, dict):
-            print("WARNING: Could not add entry points. They were:")
-            print(entry_points)
-        else:
-            cs = entry_points.get('console_scripts', [])
-            gs = entry_points.get('gui_scripts', [])
-            if isinstance(cs, string_types):
-                cs = [cs]
-            elif cs and isinstance(cs, list) and isinstance(cs[0], list):
-                # We can have lists of lists here
-                cs = [item for sublist in [s for s in cs] for item in sublist]
-            if isinstance(gs, string_types):
-                gs = [gs]
-            elif gs and isinstance(gs, list) and isinstance(gs[0], list):
-                gs = [item for sublist in [s for s in gs] for item in sublist]
-            # We have *other* kinds of entry-points so we need
-            # setuptools at run-time
-            if set(entry_points.keys()) - {'console_scripts', 'gui_scripts'}:
-                setuptools_run = True
-            # TODO: Use pythonw for gui scripts
-            entry_list = (cs + gs)
-            if len(cs + gs) != 0:
-                d['entry_points'] = entry_list
-                d['test_commands'] = make_entry_tests(entry_list)
+    d.update(get_entry_points(pkginfo))
 
     requires = get_requirements(package, pkginfo, all_extras=all_extras)
 
-    if requires or setuptools_run:
+    if requires or is_setuptools_enabled(pkginfo):
         deps = []
-        if setuptools_run:
+        if is_setuptools_enabled(pkginfo):
             deps.append('setuptools')
         for deptext in requires:
             if isinstance(deptext, string_types):
@@ -778,8 +732,6 @@ def get_package_metadata(package, d, data, output_dir, python_version, all_extra
                         spec = ' '.join((spec, marker))
                     deps.append(spec)
 
-        if 'setuptools' in deps:
-            setuptools_run = False
         d['build_depends'] = ['pip'] + deps
         # Never add setuptools to runtime dependencies.
         d['run_depends'] = deps
@@ -826,14 +778,15 @@ def get_package_metadata(package, d, data, output_dir, python_version, all_extra
         d['summary'] = "Summary of the package"
 
     license_classifier = "License :: OSI Approved :: "
-    if pkginfo.get('classifiers'):
-        licenses = [classifier.split(license_classifier, 1)[1] for
-            classifier in pkginfo['classifiers'] if classifier.startswith(license_classifier)]
-    elif data and 'classifiers' in data:
-        licenses = [classifier.split(license_classifier, 1)[1] for classifier in
-                data['classifiers'] if classifier.startswith(license_classifier)]
-    else:
-        licenses = []
+    data_classifier = data.get("classifiers", []) if data else []
+    pkg_classifier = pkginfo.get('classifiers', data_classifier)
+    pkg_classifier = pkg_classifier if pkg_classifier else data_classifier
+    licenses = [
+        classifier.split(license_classifier, 1)[1]
+        for classifier in pkg_classifier
+        if classifier.startswith(license_classifier)
+    ]
+
     if not licenses:
         if pkginfo.get('license'):
             license_name = pkginfo['license']
@@ -841,6 +794,7 @@ def get_package_metadata(package, d, data, output_dir, python_version, all_extra
             license_name = data['license']
         else:
             license_name = None
+
         if license_name:
             if noprompt:
                 pass
@@ -853,13 +807,13 @@ def get_package_metadata(package, d, data, output_dir, python_version, all_extra
                 print(license_name)
                 print()
                 license_name = input("What license string should I use? ")
+        elif noprompt:
+            license_name = "UNKNOWN"
         else:
-            if noprompt:
-                license_name = "UNKNOWN"
-            else:
-                license_name = input(("No license could be found for %s on " +
-                                    "PyPI or in the source. What license should I use? ") %
-                                package)
+            license_name = input(
+                "No license could be found for %s on PyPI or in the source. "
+                "What license should I use? " % package
+            )
     else:
         license_name = ' or '.join(licenses)
     # remove the word license from the license
@@ -870,6 +824,88 @@ def get_package_metadata(package, d, data, output_dir, python_version, all_extra
         d['digest'] = pkginfo['new_hash_value']
 
 
+def get_entry_points(pkginfo):
+    """Look at the entry_points and construct console_script and gui_scripts entry_points for conda
+    :param pkginfo:
+    :return dict:
+    """
+    entry_points = pkginfo.get('entry_points')
+    if not entry_points:
+        return {}
+
+    if isinstance(entry_points, str):
+        # makes sure it is left-shifted
+        newstr = "\n".join(x.strip() for x in entry_points.splitlines())
+        _config = configparser.ConfigParser()
+
+        try:
+            # TODO: Should we use read_file instead of readfp?
+            #  Because readfp is deprecated
+            _config.readfp(StringIO(newstr))
+        except Exception as err:
+            print("WARNING: entry-points not understood: ", err)
+            print("The string was", newstr)
+        else:
+            entry_points = {}
+            for section in _config.sections():
+                if section in ['console_scripts', 'gui_scripts']:
+                    entry_points[section] = [
+                        '%s=%s' % (option, _config.get(section, option))
+                        for option in _config.options(section)
+                    ]
+
+    if isinstance(entry_points, dict):
+        console_script = convert_to_flat_list(
+            entry_points.get('console_scripts', [])
+        )
+        gui_scripts = convert_to_flat_list(
+            entry_points.get('gui_scripts', [])
+        )
+
+        # TODO: Use pythonw for gui scripts
+        entry_list = console_script + gui_scripts
+        if entry_list:
+            return {
+                "entry_points": entry_list,
+                "test_commands": make_entry_tests(entry_list)
+            }
+    else:
+        print("WARNING: Could not add entry points. They were:")
+        print(entry_points)
+    return {}
+
+
+def convert_to_flat_list(var_scripts):
+    """Convert a string to a list.
+    If the first element of the list is a nested list this function will
+    convert it to a flat list.
+
+    :param str/list var_scripts: Receives a string or a list to be converted
+    :return list: Return a flat list
+    """
+    if isinstance(var_scripts, string_types):
+        var_scripts = [var_scripts]
+    elif var_scripts and isinstance(var_scripts, list) and isinstance(var_scripts[0], list):
+        var_scripts = [item for sublist in [s for s in var_scripts] for item in sublist]
+    return var_scripts
+
+
+def is_setuptools_enabled(pkginfo):
+    """Function responsible to inspect if skeleton requires setuptools
+    :param dict pkginfo: Dict which holds the package information
+    :return Bool: Return True if it is enabled or False otherwise
+    """
+    entry_points = pkginfo.get("entry_points")
+    if not isinstance(entry_points, dict):
+        return False
+
+    # We have *other* kinds of entry-points so we need
+    # setuptools at run-time
+    if set(entry_points.keys()) - {'console_scripts', 'gui_scripts'}:
+        return True
+    return False
+
+
 def valid(name):
     if (re.match("[_A-Za-z][_a-zA-Z0-9]*$", name) and not keyword.iskeyword(name)):
         return name
