@@ -42,77 +42,28 @@ source:
     folder: source
 
 build:
+  number: 0
+  noarch: generic
   missing_dso_whitelist:
     - '*'
 
-{depends_build}
+{depends}
 
-outputs:
-  - name: {packagename}
-    build:
-      noarch: generic
-{depends_run}
-    about:
-      home: {home}
-      license: {license}
-      license_family: {license_family}
-      summary: {summary}
-      description: {description}
+about:
+  home: {home}
+  license: {license}
+  license_family: {license_family}
+  summary: {summary}
+  description: {description}
 """
 
 
-RPM2CPIO = """\
-#!/bin/sh
-
-# Based on:
-# https://www.redhat.com/archives/rpm-list/2003-June/msg00367.html
-# Modified to also support xz compression.
-
-pkg=$1
-if [ "$pkg" = "" -o ! -e "$pkg" ]; then
-    echo "no package supplied" 1>&2
-   exit 1
-fi
-
-leadsize=96
-o=`expr $leadsize + 8`
-set `od -j $o -N 8 -t u1 $pkg`
-il=`expr 256 \* \( 256 \* \( 256 \* $2 + $3 \) + $4 \) + $5`
-dl=`expr 256 \* \( 256 \* \( 256 \* $6 + $7 \) + $8 \) + $9`
-
-sigsize=`expr 8 + 16 \* $il + $dl`
-o=`expr $o + $sigsize + \( 8 - \( $sigsize \% 8 \) \) \% 8 + 8`
-set `od -j $o -N 8 -t u1 $pkg`
-il=`expr 256 \* \( 256 \* \( 256 \* $2 + $3 \) + $4 \) + $5`
-dl=`expr 256 \* \( 256 \* \( 256 \* $6 + $7 \) + $8 \) + $9`
-
-hdrsize=`expr 8 + 16 \* $il + $dl`
-o=`expr $o + $hdrsize`
-
-hdr=`dd if=$pkg ibs=$o skip=1 count=1 2>/dev/null | od -N 2 -t x1 -An`
-# macOS dd and Linux od give different results
-hdr="${hdr#"${hdr%%[![:space:]]*}"}"
-# remove trailing whitespace characters
-hdr="${hdr%"${hdr##*[![:space:]]}"}"
-if [[ "$hdr" == "1f 8b" ]] || [[ "$hdr" == "1f  8b" ]]; then
-  dd if=$pkg ibs=$o skip=1 2>/dev/null | gunzip
-else
-  dd if=$pkg ibs=$o skip=1 2>/dev/null | xz -d
-fi
-"""
-
 BUILDSH = """\
 #!/bin/bash
 
-RPM=$(find ${PWD}/binary -name "*.rpm")
-mkdir -p ${PREFIX}/x86_64-conda_cos6-linux-gnu/sysroot/usr
-pushd ${PREFIX}/x86_64-conda_cos6-linux-gnu/sysroot/usr > /dev/null 2>&1
-if [[ -n "${RPM}" ]]; then
-  "${RECIPE_DIR}"/rpm2cpio "${RPM}" | cpio -idmv
-  popd > /dev/null 2>&1
-else
-  cp -Rf "${SRC_DIR}"/binary/* .
-fi
+mkdir -p ${PREFIX}/x86_64-conda_cos6-linux-gnu/sysroot
+pushd ${PREFIX}/x86_64-conda_cos6-linux-gnu/sysroot > /dev/null 2>&1
+cp -Rf "${SRC_DIR}"/binary/* .
 """
 
 
@@ -329,13 +280,13 @@ def get_repo_dict(repomd_url, data_type, dict_massager, cdt, src_cache):
         open_csum = child.findall("open-checksum")[0].text
         xml_file = join(src_cache, open_csum)
         try:
-            xml_file, xml_csum = cache_file(src_cache, xml_file, cdt['checksummer'])
+            xml_file, xml_csum = cache_file(src_cache, xml_file, None, cdt['checksummer'])
         except:
             csum = child.findall("checksum")[0].text
             location = child.findall("location")[0].attrib['href']
             xmlgz_file = dirname(dirname(repomd_url)) + '/' + location
             cached_path, cached_csum = cache_file(src_cache, xmlgz_file,
-                                                  csum, cdt['checksummer'])
+                                                  None, cdt['checksummer'])
             assert csum == cached_csum, "Checksum for {} does not match value in {}".format(
                 xmlgz_file, repomd_url)
             with gzip.open(cached_path, 'rb') as gz:
@@ -549,6 +500,7 @@ def write_conda_recipes(recursive, repo_primary, package, architectures,
                                                  src_cache)
 
     sn = cdt['short_name'] + '-' + arch
+    dependsstr = ""
     if len(depends):
         depends_specs = ["{}-{}-{} {}{}".format(depend['name'].lower().replace('+', 'x'),
                                                 cdt['short_name'], depend['arch'],
@@ -556,13 +508,9 @@ def write_conda_recipes(recursive, repo_primary, package, architectures,
                          for depend in depends]
         dependsstr_part = '\n'.join(['    - {}'.format(depends_spec)
                                      for depends_spec in depends_specs])
-        dependsstr_build = 'requirements:\n' \
-                           '  build:\n' + dependsstr_part + '\n'
-        dependsstr_run = '    requirements:\n' \
-                           '      run:\n    ' + '\n    '.join(dependsstr_part.split('\n')) + '\n'
-    else:
-        dependsstr_build = ''
-        dependsstr_run = ''
+        dependsstr_build = '  build:\n' + dependsstr_part + '\n'
+        dependsstr_run = '  run:\n' + dependsstr_part
+        dependsstr = 'requirements:\n' + dependsstr_build + dependsstr_run
 
     package_l = package.lower().replace('+', 'x')
     package_cdt_name = package_l + '-' + sn
@@ -571,8 +519,7 @@ def write_conda_recipes(recursive, repo_primary, package, architectures,
               'packagename': package_cdt_name,
               'hostmachine': cdt['host_machine'],
               'hostsubdir': cdt['host_subdir'],
-              'depends_build': dependsstr_build,
-              'depends_run': dependsstr_run,
+              'depends': dependsstr,
               'rpmurl': rpm_url,
               'srcrpmurl': srpm_url,
               'home': entry['home'],
@@ -599,10 +546,6 @@ def write_conda_recipes(recursive, repo_primary, package, architectures,
         pass
     with open(join(odir, 'meta.yaml'), 'w') as f:
         f.write(RPM_META.format(**d))
-    rpm2cpio = join(odir, 'rpm2cpio')
-    with open(rpm2cpio, 'w') as f:
-        chmod(rpm2cpio, 0o755)
-        f.write(RPM2CPIO)
     buildsh = join(odir, 'build.sh')
     with open(buildsh, 'w') as f:
         chmod(buildsh, 0o755)
