@@ -23,9 +23,16 @@ import tarfile
 import tempfile
 from threading import Thread
 import time
+
+try:
+    from json.decoder import JSONDecodeError
+except ImportError:
+    JSONDecodeError = ValueError
+
 import yaml
 
 import filelock
+import conda_package_handling.api
 
 try:
     from conda.base.constants import CONDA_TARBALL_EXTENSIONS
@@ -33,6 +40,8 @@ except Exception:
     from conda.base.constants import CONDA_TARBALL_EXTENSION
     CONDA_TARBALL_EXTENSIONS = (CONDA_TARBALL_EXTENSION,)
 
+from conda.api import PackageCacheData
+
 from .conda_interface import hashsum_file, md5_file, unix_path_to_win, win_path_to_unix
 from .conda_interface import PY3, iteritems
 from .conda_interface import root_dir, pkgs_dirs
@@ -41,8 +50,9 @@ from .conda_interface import memoized
 from .conda_interface import StringIO
 from .conda_interface import VersionOrder, MatchSpec
 from .conda_interface import cc_conda_build
-from .conda_interface import conda_43, Dist
+from .conda_interface import conda_43, conda_46, Dist
 from .conda_interface import context
+from .conda_interface import download, TemporaryDirectory, get_conda_channel, CondaHTTPError
 # NOQA because it is not used in this file.
 from conda_build.conda_interface import rm_rf as _rm_rf # NOQA
 from conda_build.exceptions import BuildLockError
@@ -83,6 +93,19 @@ mmap_MAP_PRIVATE = 0 if on_win else mmap.MAP_PRIVATE
 mmap_PROT_READ = 0 if on_win else mmap.PROT_READ
 mmap_PROT_WRITE = 0 if on_win else mmap.PROT_WRITE
 
+DEFAULT_SUBDIRS = {
+    "linux-64",
+    "linux-32",
+    "linux-ppc64le",
+    "linux-armv6l",
+    "linux-armv7l",
+    "linux-aarch64",
+    "win-64",
+    "win-32",
+    "osx-64",
+    "zos-z",
+    "noarch",
+}
 
 PY_TMPL = """
 # -*- coding: utf-8 -*-
@@ -560,6 +583,18 @@ def copy_into(src, dst, timeout=900, symlinks=False, lock=None, locking=True, cl
                             os.path.basename(src), dst)
 
 
+def move_with_fallback(src, dst):
+    try:
+        shutil.move(src, dst)
+    except PermissionError:
+        copy_into(src, dst)
+        try:
+            os.unlink(src)
+        except PermissionError:
+            log = get_logger(__name__)
+            log.debug("Failed to clean up temp path due to permission error: %s" % src)
+
+
 # http://stackoverflow.com/a/22331852/1170370
 def copytree(src, dst, symlinks=False, ignore=None, dry_run=False):
     if not os.path.exists(dst):
@@ -703,7 +738,77 @@ def relative(f, d='lib'):
 # This is the lowest common denominator of the formats supported by our libarchive/python-libarchive-c
 # packages across all platforms
 decompressible_exts = ('.7z', '.tar', '.tar.bz2', '.tar.gz', '.tar.lzma', '.tar.xz',
-                       '.tar.z', '.tgz', '.whl', '.zip', '.rpm', '.deb')
+                       '.tar.z', '.tar.zst', '.tgz', '.whl', '.zip', '.rpm', '.deb')
+
+
+def _tar_xf_fallback(tarball, dir_path, mode='r:*'):
+    if tarball.lower().endswith('.tar.z'):
+        uncompress = external.find_executable('uncompress')
+        if not uncompress:
+            uncompress = external.find_executable('gunzip')
+        if not uncompress:
+            sys.exit("""\
+uncompress (or gunzip) is required to unarchive .z source files.
+""")
+        check_call_env([uncompress, '-f', tarball])
+        tarball = tarball[:-2]
+    if not PY3 and tarball.endswith('.tar.xz'):
+        unxz = external.find_executable('unxz')
+        if not unxz:
+            sys.exit("""\
+unxz is required to unarchive .xz source files.
+""")
+
+        check_call_env([unxz, '-f', '-k', tarball])
+        tarball = tarball[:-3]
+    t = tarfile.open(tarball, mode)
+    members = t.getmembers()
+    for i, member in enumerate(members, 0):
+        if os.path.isabs(member.name):
+            member.name = os.path.relpath(member.name, '/')
+        if not os.path.realpath(member.name).startswith(os.getcwd()):
+            member.name = member.name.replace("../", "")
+        if not os.path.realpath(member.name).startswith(os.getcwd()):
+            sys.exit("tarball contains unsafe path: " + member.name)
+        members[i] = member
+
+    if not PY3:
+        t.extractall(path=dir_path.encode(codec))
+    else:
+        t.extractall(path=dir_path)
+    t.close()
+
+
+def tar_xf_file(tarball, entries):
+    from conda_build.utils import ensure_list
+    entries = ensure_list(entries)
+    if not os.path.isabs(tarball):
+        tarball = os.path.join(os.getcwd(), tarball)
+    result = None
+    n_found = 0
+    with libarchive.file_reader(tarball) as archive:
+        for entry in archive:
+            if entry.name in entries:
+                n_found += 1
+                for block in entry.get_blocks():
+                    if result is None:
+                        result = bytes(block)
+                    else:
+                        result += block
+                break
+    if n_found != len(entries):
+        raise KeyError()
+    return result
+
+
+def tar_xf_getnames(tarball):
+    if not os.path.isabs(tarball):
+        tarball = os.path.join(os.getcwd(), tarball)
+    result = []
+    with libarchive.file_reader(tarball) as archive:
+        for entry in archive:
+            result.append(entry.name)
+    return result
 
 
 def tar_xf(tarball, dir_path):
@@ -714,8 +819,17 @@ def tar_xf(tarball, dir_path):
             libarchive.extract.EXTRACT_SECURE_NOABSOLUTEPATHS
     if not os.path.isabs(tarball):
         tarball = os.path.join(os.getcwd(), tarball)
-    with tmp_chdir(dir_path):
-        libarchive.extract_file(tarball, flags)
+    try:
+        with tmp_chdir(dir_path):
+            libarchive.extract_file(tarball, flags)
+    except libarchive.exception.ArchiveError:
+        # try again, maybe we are on Windows and the archive contains symlinks
+        # https://github.com/conda/conda-build/issues/3351
+        # https://github.com/libarchive/libarchive/pull/1030
+        if tarball.endswith(('.tar', '.tar.gz', '.tgz', '.tar.bz2', '.tar.z', '.tar.xz')):
+            _tar_xf_fallback(tarball, dir_path)
+        else:
+            raise
 
 
 def file_info(path):
@@ -1000,24 +1114,37 @@ def get_skip_message(m):
         {k: m.config.variant[k] for k in m.get_used_vars()}))
 
 
-@memoized
-def package_has_file(package_path, file_path):
-    try:
-        locks = get_conda_operation_locks()
-        with try_acquire_locks(locks, timeout=900):
-            with tarfile.open(package_path) as t:
-                try:
-                    # internal paths are always forward slashed on all platforms
-                    file_path = file_path.replace('\\', '/')
-                    text = t.extractfile(file_path).read()
-                    return text
-                except KeyError:
-                    return False
-                except OSError as e:
-                    raise RuntimeError("Could not extract %s (%s)" % (package_path, e))
-    except (tarfile.ReadError, IOError, EOFError):
-        raise RuntimeError("Could not extract metadata from %s. File probably corrupt.  "
-                           "Please manually remove this file and try again." % package_path)
+def package_has_file(package_path, file_path, refresh=False):
+    locks = get_conda_operation_locks()
+    possible_subdir = os.path.basename(os.path.dirname(package_path))
+    possible_subdir = possible_subdir if possible_subdir in DEFAULT_SUBDIRS else ''
+    with try_acquire_locks(locks, timeout=900):
+        folder_name = os.path.basename(conda_package_handling.api.get_default_extracted_folder(package_path))
+        # look in conda's package cache
+        try:
+            # conda 4.7.2 added this
+            cache_path = PackageCacheData.first_writable().pkgs_dir
+        except AttributeError:
+            # fallback; assume writable first path.  Not as reliable.
+            cache_path = pkgs_dirs[0]
+        cache_path = os.path.join(cache_path, possible_subdir) if possible_subdir else cache_path
+        cache_path = os.path.join(cache_path, folder_name)
+        resolved_file_path = os.path.join(cache_path, file_path)
+        if not os.path.isfile(resolved_file_path) or refresh:
+            if file_path.startswith('info'):
+                conda_package_handling.api.extract(package_path, cache_path, 'info')
+            else:
+                conda_package_handling.api.extract(package_path, cache_path)
+        if not os.path.isfile(resolved_file_path):
+            return False
+        else:
+            try:
+                with open(resolved_file_path) as f:
+                    content = f.read()
+            except UnicodeDecodeError:
+                with open(resolved_file_path, 'rb') as f:
+                    content = f.read()
+    return content
 
 
 def ensure_list(arg):
@@ -1059,7 +1186,7 @@ def expand_globs(path_list, root_dir):
             files.extend(glob_files)
     prefix_path_re = re.compile('^' + re.escape('%s%s' % (root_dir, os.path.sep)))
     files = [prefix_path_re.sub('', f, 1) for f in files]
-    return sorted(files)
+    return files
 
 
 def find_recipe(path):
@@ -1286,7 +1413,8 @@ def filter_files(files_list, prefix, filter_patterns=('(.*[\\\\/])?\.git[\\\\/].
                                                       '(.*[\\\\/])?\.git$',
                                                       '(.*)?\.DS_Store.*',
                                                       '.*\.la$',
-                                                      'conda-meta.*')):
+                                                      'conda-meta.*',
+                                                      '.*\.conda_trash(?:_\d+)*$')):
     """Remove things like the .git directory from the list of files to be copied"""
     for pattern in filter_patterns:
         r = re.compile(pattern)
@@ -1312,7 +1440,8 @@ def filter_info_files(files_list, prefix):
                     'info[\\\\/]recipe_log.json',
                     'info[\\\\/]recipe.tar',
                     'info[\\\\/]test[\\\\/].*',
-                    'info[\\\\/]LICENSE.txt',
+                    'info[\\\\/]LICENSE.txt',        # legacy, some tests rely on this
+                    'info[\\\\/]licenses[\\\\/]*',
                     'info[\\\\/]requires',
                     'info[\\\\/]meta',
                     'info[\\\\/]platform',
@@ -1323,6 +1452,8 @@ def filter_info_files(files_list, prefix):
 
 
 def rm_rf(path, config=None):
+    if conda_46:
+        return _rm_rf(path)
     if os.path.isdir(path):
         try:
             # subprocessing to delete large folders can be quite a bit faster
@@ -1461,7 +1592,7 @@ def merge_or_update_dict(base, new, path="", merge=True, raise_on_clobber=False,
                     if base_value != value:
                         try:
                             base_value.extend(value)
-                        except TypeError:
+                        except (TypeError, AttributeError):
                             base_value = value
                     try:
                         base[key] = list(base_value)
@@ -1495,14 +1626,16 @@ def prefix_files(prefix):
     Returns a set of all files in prefix.
     '''
     res = set()
+    prefix_rep = prefix + os.path.sep
     for root, dirs, files in walk(prefix):
         for fn in files:
-            res.add(join(root, fn)[len(prefix) + 1:])
+            # this is relpath, just hacked to be faster
+            res.add(join(root, fn).replace(prefix_rep, '', 1))
         for dn in dirs:
             path = join(root, dn)
             if islink(path):
-                res.add(path[len(prefix) + 1:])
-    res = set(expand_globs(res, prefix))
+                res.add(path.replace(prefix_rep, '', 1))
+                res.update(expand_globs((path, ), prefix))
     return res
 
 
@@ -1686,3 +1819,81 @@ def sha256_checksum(filename, buffersize=65536):
         for block in iter(lambda: f.read(buffersize), b''):
             sha256.update(block)
     return sha256.hexdigest()
+
+
+def write_bat_activation_text(file_handle, m):
+    if conda_46:
+        file_handle.write('call "{conda_root}\\..\\condabin\\conda_hook.bat"\n'.format(
+            conda_root=root_script_dir,
+        ))
+    if m.is_cross:
+        # HACK: we need both build and host envs "active" - i.e. on PATH,
+        #     and with their activate.d scripts sourced. Conda only
+        #     lets us activate one, though. This is a
+        #     vile hack to trick conda into "stacking"
+        #     two environments.
+        #
+        # Net effect: binaries come from host first, then build
+        #
+        # Conda 4.4 may break this by reworking the activate scripts.
+        #  ^^ shouldn't be true
+        # In conda 4.4, export CONDA_MAX_SHLVL=2 to stack envs to two
+        #   levels deep.
+        # conda 4.4 does require that a conda-meta/history file
+        #   exists to identify a valid conda environment
+        # conda 4.6 changes this one final time, by adding a '--stack' flag to the 'activate'
+        #   command, and 'activate' does not stack environments by default without that flag
+        history_file = join(m.config.host_prefix, 'conda-meta', 'history')
+        if not isfile(history_file):
+            if not isdir(dirname(history_file)):
+                os.makedirs(dirname(history_file))
+            open(history_file, 'a').close()
+
+        if conda_46:
+            file_handle.write('call "{conda_root}\\..\\condabin\\conda.bat" activate "{prefix}"\n'.format(
+                conda_root=root_script_dir,
+                prefix=m.config.host_prefix,
+            ))
+        else:
+            file_handle.write('call "{conda_root}\\activate.bat" "{prefix}"\n'.format(
+                conda_root=root_script_dir,
+                prefix=m.config.host_prefix))
+            # removing this placeholder should make conda double-activate with conda 4.3
+            file_handle.write('set "PATH=%PATH:CONDA_PATH_PLACEHOLDER;=%"\n')
+            file_handle.write('set CONDA_MAX_SHLVL=2\n')
+
+    # Write build prefix activation AFTER host prefix, so that its executables come first
+    if conda_46:
+        file_handle.write('call "{conda_root}\\..\\condabin\\conda.bat" activate --stack "{prefix}"\n'.format(
+            conda_root=root_script_dir,
+            prefix=m.config.build_prefix,
+        ))
+    else:
+        file_handle.write('call "{conda_root}\\activate.bat" "{prefix}"\n'.format(
+            conda_root=root_script_dir,
+            prefix=m.config.build_prefix))
+
+
+channeldata_cache = {}
+
+
+def download_channeldata(channel_url):
+    global channeldata_cache
+    if channel_url.startswith('file://') or channel_url not in channeldata_cache:
+        urls = get_conda_channel(channel_url).urls()
+        urls = set(url.rsplit('/', 1)[0] for url in urls)
+        data = {}
+        for url in urls:
+            with TemporaryDirectory() as td:
+                tf = os.path.join(td, "channeldata.json")
+                try:
+                    download(url + '/channeldata.json', tf)
+                    with open(tf) as f:
+                        new_channeldata = json.load(f)
+                except (JSONDecodeError, CondaHTTPError):
+                    new_channeldata = {}
+            merge_or_update_dict(data, new_channeldata)
+        channeldata_cache[channel_url] = data
+    else:
+        data = channeldata_cache[channel_url]
+    return data
